#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var evidences_1 = require("./evidences");
var references_1 = require("./references");
var signatures_1 = require("./signatures");
var simple_segment_1 = require("./simple_segment");
/** TestCases to run. */
var TestCases = new Array(new simple_segment_1.SimpleSegmentTest(), new references_1.ReferencesTest(), new evidences_1.EvidencesTest(), new signatures_1.SignaturesTest());
/**
 * Execute commands to produce or validate chainscript data.
 * @param action to execute.
 * @param path to read/write segments.
 */
function run(action, path) {
    switch (action) {
        case "generate":
            return generate(path);
        case "validate":
            return validate(path);
        default:
            console.log("Unknown action " + action);
            process.exit(1);
    }
}
/**
 * Generate encoded test segments.
 * @param path to the file where test segments will be written.
 */
function generate(path) {
    var results = TestCases.map(function (t) {
        return { id: t.id(), data: t.generate() };
    });
    console.log("Saving encoded segments to " + path + "...");
    fs_1.writeFile(path, JSON.stringify(results), function (err) {
        if (err) {
            console.error(err);
            process.exit(1);
        }
        else {
            console.log("Saved.");
        }
    });
}
/**
 * Validate encoded test segments.
 * @param path to the file containing the test segments.
 */
function validate(path) {
    console.log("Loading encoded segments from " + path + "...");
    fs_1.readFile(path, function (err, data) {
        if (err) {
            console.log(err);
            process.exit(1);
        }
        var testData = JSON.parse(data.toString());
        var failed = false;
        testData.map(function (t) {
            var test;
            switch (t.id) {
                case simple_segment_1.SimpleSegmentTest.id:
                    test = new simple_segment_1.SimpleSegmentTest();
                    break;
                case references_1.ReferencesTest.id:
                    test = new references_1.ReferencesTest();
                    break;
                case evidences_1.EvidencesTest.id:
                    test = new evidences_1.EvidencesTest();
                    break;
                case signatures_1.SignaturesTest.id:
                    test = new signatures_1.SignaturesTest();
                    break;
                default:
                    console.log("Unknown test case: " + t.id);
                    failed = true;
                    return;
            }
            try {
                test.validate(t.data);
                console.log("[" + t.id + "] SUCCESS");
            }
            catch (err) {
                console.log("[" + t.id + "] FAILED: " + err);
                failed = true;
            }
        });
        if (failed) {
            process.exit(1);
        }
    });
}
// Run the test command.
run(process.argv[2], process.argv[3]);
//# sourceMappingURL=cli.js.map